#Define tests (as an array of in-groups, with species to test having the same value in the second column of the array)
test<-c('Obi',3,1,
  'Obi',2,1,
  'Lgi',2,1,
  'Cgi',2,1,
  'Pfu',2,1,
  'Obi',1,1,
  'Lgi',1,1,
  'Cgi',1,1,
  'Pfu',1,1,
  'Cte',1,1,
  'Hro',1,1
  )

m<-matrix(test,ncol=3,byrow=T)
m=m[order(m[,2]),]

TOTAL=unique(m[,2])

excl=c() # if any species need to be excluded
data = read.table("pca.pfam",sep="\t",row.names=1,header=T)
data=data[,!(colnames(data) %in% excl)]

#exclude transposons etc
data = data[!grepl("unknown",rownames(data),ignore.case=T),]
data = data[!grepl("not named",rownames(data),ignore.case=T),]
data = data[!grepl("uncharacterized",rownames(data),ignore.case=T),]
data = data[!grepl("transposase",rownames(data),ignore.case=T),]
data = data[!grepl("helitron",rownames(data),ignore.case=T),]
data = data[!grepl("helicase",rownames(data),ignore.case=T),]
data = data[!grepl("DUF",rownames(data),ignore.case=T),]
data = data[!grepl("DDE_Tnp",rownames(data),ignore.case=T),]
data = data[!grepl("PF14529 Exo_endo_phos_2",rownames(data),ignore.case=T),]

ORD<-NULL

#merge some PFAMs - after manual assessment
merge1=c("PF08266 Cadherin_2","PF00028 Cadherin")
merge2=c("PF13465 zf-H2C2_2","PF00096 zf-C2H2")
merge3=c("PF02868 Peptidase_M4_C","PF01447 Peptidase_M4")
merge=c(merge1,merge2,merge3)
data0=data
data=data0[!(rownames(data0) %in% merge),]

data<-rbind(data, colSums(data0[rownames(data0) %in% merge1,]))
rownames(data)[length(data[,1])]="PF08266/00028 Cadherin"
data<-rbind(data, colSums(data0[rownames(data0) %in% merge2,]))
rownames(data)[length(data[,1])]="PF13465/00096 zf-C2H2"
data<-rbind(data, colSums(data0[rownames(data0) %in% merge3,]))
rownames(data)[length(data[,1])]="PF02868/01447 Peptidase_M4"



for (i in TOTAL) {
 print(i)
 #store all in-group species
 col1=m[m[,2]==i&m[,3]==1,1]
 #get all out-group species
 col2=colnames(data)[!colnames(data) %in% col1]
 col2b=m[m[,2]==i&m[,3]==2,1]
 #show them
 print(col1)
 print(col2)
 
 tmpord=rownames(data)
 for (X in col1) { # test individual in-group species against the total of the outgroups
 print(X)
 a=as.matrix(data[,X])
 b=as.matrix(data[,col2])
 af<-NULL
 bf<-NULL
 for (j in c(1:length(a[,1]))) {
  af<-rbind(af,sum(a[j,])) #total in-group
  bf<-rbind(bf,sum(b[j,])) #total outgroup
 }
 fm=cbind(af,bf)
 rownames(fm)<-rownames(data)

 totala=sum(fm[,1])
 totalb=sum(fm[,2])
 pval<-NULL
 for (ii in c(1:length(fm[,1]))) {
 pval0 <- NULL
 m0 <- matrix(c(fm[ii,1],totala-fm[ii,1],fm[ii,2],totalb-fm[ii,2]),nrow=2,ncol=2)
 pval0 = c(pval0,fisher.test(m0,alternative="greater")$p)
 pval<-rbind(pval,pval0);
}

pval2<-p.adjust(pval,method="bonferroni") #multiple testing correction


tmpord=cbind(tmpord,pval2)


} # end of loop through in-group species

tmpord2=NULL
for (iii in c(1:length(tmpord[,1]))) {
 if (length(col1)==1) { minn=1 } else { minn=length(col1)/2 }
 # check if at least half of the in-group species are significantly expanded
 if (sum(as.numeric(tmpord[iii,2:(length(col1)+1)])<=0.01)>(length(col1)/2)) {  # length(col1)/2
 print(as.numeric(tmpord[iii,2:(length(col1)+1)]))
 tmpord2=rbind(tmpord2,c(tmpord[iii,1],median(as.numeric(tmpord[iii,2:(length(col1)+1)]))))
}
}
print(tmpord2)
if (!(is.null(tmpord2))) {
tmpord=matrix(tmpord2[!(tmpord2[,1] %in% ORD[,1]),],ncol=2)
tmpord=tmpord[order(tmpord[,2]),]
ORD<-rbind(ORD,tmpord)
}
}


res<-NULL
normvec<-NULL
for (i in c(1:length(data[1,]))) {
 normvec<-c(normvec,sum(data[,i]))
}

ORD2=rev(ORD[,1])
# normalize for total gene count for plotting
for (i in c(1:length(ORD2))) {
 res<-rbind(res,data[ORD2[i],]/normvec)
}

library(RColorBrewer)
res=res[,c("Obi","Lgi","Pfu","Cgi","Cte","Hro","Dme","Cel","Bfl","Dre","Lch","Xtr","Gga","Mmu","Hsa")]

library(gplots)


pal=rownames(brewer.pal.info)
pal="Blues"
for (i in pal) {
heatmap.2(as.matrix(res),scale='row',Colv=FALSE,Rowv=FALSE,dendrogram="none",trace="none",margins=c(10,15),density.info="none",sepwidth=c(0,0),col=brewer.pal(9,i))
}


